/*
 * =====================================================================================
 *
 *       Filename:  main.cpp
 *
 *    Description:  Main file
 *
 *        Version:  1.0
 *        Created:  11/01/2014 16:56:37
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Daohang Shi, Jackson Melchert
 *
 * =====================================================================================
 */

#include <cstdio>
#include <sys/param.h>
#include <sys/time.h>
#include <cstdlib>
#include <cstring>
#include <string>

#include "parser.h"


int main( int argc, char ** argv ) 
{

    char *  designName = nullptr;
    char *  auxFile = nullptr;
    char *  rtFile = nullptr;
    int i = 1;

    // Read input arguments
    while( i < argc ) {
        if( !strcmp( argv[i], "-design" ) ) {
            i++;
            assert( i < argc );
            designName = argv[i++];
        } else if( !strcmp( argv[i], "-auxFile" ) ) {
            i++;
            assert( i < argc );
            auxFile = argv[i++];
        } else if( !strcmp( argv[i], "-rtFile" ) ) {
            i++;
            assert( i < argc );
            rtFile = argv[i++];
        } else {
            cout << "Usage : ./readBenchmark -design <design_name> -auxFile <aux_file> -rtFile <rt_file>\n";
            exit(0);
        }
    }

    if( !designName || !auxFile || !rtFile ) {
        cout << "Usage : ./readBenchmark -design <design_name> -auxFile <aux_file> -rtFile <rt_file>\n";
        exit(1);
    }

    if( designName )    cout << "Design : " << designName << endl;

    Parser parser;
    Layout layout;

    layout.initDesignName( designName );

    // Read design
    parser.ReadAux( auxFile );
    parser.ReadNode( layout );
    parser.ReadPlace( layout );
    parser.ReadShape( layout );
    parser.ReadRouteConfig( layout );
    parser.ReadNet( layout );

    RoutingDB routingDB( designName, layout );

    routingDB.initGlobalNets( layout );

    routingDB.initEdges( layout );
    routingDB.initGlobalEdgeProfile( layout );

    // Read routing file
    if( rtFile ) {

        routingDB.readGlobalWires( layout, rtFile );

        routingDB.initRoutingTreeForAllGnets();

        routingDB.updateEdgeDemands();

    }

    return 0;
}
