#ifndef _NETS_H_
#define _NETS_H_

#include "baseDB.h"

#include <iostream>
#include <string>
#include <sstream>
#include <vector>

typedef Point3D NetPin;
typedef Gcell GnetPin;

class Net
{
    friend class Layout;
    friend class RoutingDB;
    public:
        Net() {}
        int init( const string & line ) {
            _netInfoLines.push_back( line );
            istringstream ss( line );
            string tmp;
            int numPins;
            ss >> tmp;
            ss >> tmp;
            ss >> numPins;
            _netPins.reserve( numPins );
            ss >> _name;
            return numPins;
        }
        string getName() const { return _name; }
        int numPins() const { return _netPins.size(); }
        vector<string>  infoLines() const { return _netInfoLines; }

    protected:
        string _name;
        vector<NetPin> _netPins;

        vector<string>  _netInfoLines;

};

class Gnet 
{
    friend class RoutingDB;
    public:
        Gnet() : _isGlobal( false ) {}
        bool isGlobal() const { return _isGlobal; }
		bool isRealNet() const { return _isRealNet; }
        int parent( const int curr ) const { return _gWires[curr]._pWireId; }
        int parent2d( const int curr ) const { return _wire2d[curr]._pWireId; } 

    private:

        //  add point pair
        void addPtPair( const Gcell & gp1, const Gcell & gp2 ) {
            _gpPairArray.push_back( make_pair( gp1, gp2 ) );
        }
        bool _isGlobal;
		bool _isRealNet;
        vector<GnetPin> _gPins;
        vector<GnetPin> _gPins2D;
        string _name;

        //  read from global wires
        vector<pair<Gcell, Gcell> > _gpPairArray;
        vector<GlobalWire> _gWires;

        //  convert from global wires to 2D wires
        vector<Wire2d> _wire2d;
};


#endif
