#ifndef _NODES_H_
#define _NODES_H_

#include "baseDB.h"
#include <iostream>
#include <cmath>
#include <string>
#include <sstream>
#include <vector>

class Node
{
    public:
        Node() : _isTerm( false ), _isNonRect( false ), _layerId(0) {}
        Node( const string & line ) { init( line ); }
        Node( const string & objName, const double width, const double height, const double llx, const double lly, const int z ) : 
            _name( objName ), _width( width ), _height( height ), _layerId( z ), _llx( llx ), _lly( lly ), _isTerm(false), _isNonRect(false) { 
                _blockedLayers.push_back( z ); 
            }

        void init( const string & line )
        {
            istringstream ss( line );
            ss >> _name;
            ss >> _width;
            ss >> _height;
        }

        void readPlace( const string & line )
        {
            istringstream ss( line );
            string name;
            ss >> name;
            ss >> _llx;
            ss >> _lly;
            _layerId = 0;
        }

        void addShape( const string & line )
        {
            istringstream ss( line );
            string tmp;
            ss >> tmp;
            double llx, lly, width, height;
            ss >> llx;
            ss >> lly;
            ss >> width;
            ss >> height;
            _shapes.push_back( Shape( llx, lly, width, height ) );
        }

        void setLayer( const unsigned layerId ) { _layerId = layerId; }
        void setIsNonRect() { _isNonRect = true; }

        void setBlockedLayers( const string & line )
        {
            istringstream ss( line );
            string tmp;
            ss >> tmp;
            unsigned num, layer;
            ss >> num;
            for( unsigned i = 0; i < num; i ++ ) {
                ss >> layer;
                _blockedLayers.push_back( layer-1 );
            }
        }

        string getName() const { return _name; }
        double getPinX( double offsetX ) const { return double( ceil( _llx + _width/2 + offsetX ) ); }
        double getPinY( double offsetY ) const { return double( ceil( _lly + _height/2 + offsetY ) ); }
        unsigned getLayerId() const { return _layerId; }
        bool isNonRect() const { return _isNonRect; }
        double getLLX() const { return _llx; }
        double getLLY() const { return _lly; }
        double getWidth() const { return _width; }
        double getHeight() const { return _height; }

        void printNode() const {
            if( _blockedLayers.empty() )
            cout << "<D> " << _name << " Regular Blockage" << endl;
            else {
                if( _isNonRect )
                    cout << "<D> " << _name << " Non-rect Blockage" << endl; 
                else
                    cout << "<D> " << _name << " Rect Blockage" << endl; 
            }
        }

    private:
        string _name;
        double _width, _height;
        bool _isTerm;
        double _llx, _lly;
        unsigned _layerId;
        //  non-rect
        bool _isNonRect;
    public:
        vector<Shape> _shapes;
        //  block
        vector<int> _blockedLayers;
};

#endif
